/*
 *  This file is part of Dune Legacy.
 *
 *  Dune Legacy is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dune Legacy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dune Legacy.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <misc/string_util.h>

#include <stdarg.h>
#include <stdio.h>

/**
	Splits a string into several substrings. This strings are separated with ','.
        Example:<br>
        String first, second;<br>
        SplitString("abc,xyz",2,&first, &second);<br>
	\param ParseString  the string to parse
	\param NumStringPointers    the number of pointers to strings following after this parameter
	\return true if successful, false otherwise.
*/
bool SplitString(std::string ParseString, unsigned int NumStringPointers, ...) {
	va_list arg_ptr;
	va_start(arg_ptr, NumStringPointers);

	std::string** pStr;

	if(NumStringPointers == 0)
		return false;

    pStr = new std::string*[NumStringPointers];

	for(unsigned int i = 0; i < NumStringPointers; i++) {
		pStr[i] = va_arg(arg_ptr, std::string* );
	}
	va_end(arg_ptr);

	int startpos = 0;
	unsigned int index = 0;

	for(unsigned int i = 0; i < ParseString.size(); i++) {
		if(ParseString[i] == ',') {
			*(pStr[index]) = ParseString.substr(startpos,i-startpos);
			startpos = i + 1;
			index++;
			if(index >= NumStringPointers) {
				delete [] pStr;
				return false;
			}
		}
	}

	*(pStr[index]) = ParseString.substr(startpos,ParseString.size()-startpos);
	delete [] pStr;
	return true;
}

/*
	Splits a string into several substrings. This strings are separated with ','.
*/
std::vector<std::string> SplitString(std::string ParseString) {
	std::vector<std::string> retVector;
	int startpos = 0;

	for(unsigned int i = 0; i < ParseString.size(); i++) {
		if(ParseString[i] == ',') {
			retVector.push_back(ParseString.substr(startpos,i-startpos));
			startpos = i + 1;
		}
	}

	retVector.push_back(ParseString.substr(startpos,ParseString.size()-startpos));
	return retVector;
}

std::string strprintf(const char* fmt, ...)
{
	va_list arg_ptr;
	va_start(arg_ptr, fmt);

	int length = vsnprintf(NULL, 0, fmt, arg_ptr);
	if(length < 0) {
        throw std::runtime_error("strprintf(): vsnprintf() failed!");
	}

	char* tmpBuffer = new char[length+1];

    va_start(arg_ptr, fmt);
	if(vsnprintf(tmpBuffer, length+1, fmt, arg_ptr) < 0) {
        throw std::runtime_error("strprintf(): vsnprintf() failed!");
	}

	std::string formatedString(tmpBuffer);

	delete [] tmpBuffer;

	return formatedString;
}
